//FAT16-Treiber

#include <P89C51Rx2.h>
#include "version.h"
#include "i2c.h"
#include "tools.h"
#include "mydefs.h"
#include "ide.h"
#include "rs232.h"


unsigned char fat_clustersize;		//Gibt die Anzahl der Sektoren pro Cluster an
unsigned long fat_start;			//Sektornummer an der die FAT beginnt
unsigned int fat_lenght;			//Gibt die Lnge _einer_ FAT an
unsigned long fat_datastart;		//Gibt den Start des data-Bereich an
unsigned long fat_rootstart;		//Gibt den Start des root-Verzeichnis an

unsigned int file_cluster = 0;		//Gibt den Cluster der Datendatei an
unsigned long file_size = 0;		//Gibt die Gre der Datendatei an
unsigned long file_pos;				//Gibt die aktuelle Position in der Datendatei an
unsigned int file_clpos;			//Gibt die Schreibposition im aktuelle Cluster an

/********************************************************
Schreibt einen 32Byte-String auf die Platte 
(512Byte Puffer)
*********************************************************/
unsigned char ide_writedatafile(unsigned char xdata *dat)
{
	unsigned char i;
	unsigned int j;
	
	if(file_clpos==(fat_clustersize*512))
	{
		file_cluster = ide_getnextcluster(file_cluster);
		file_clpos = 0;
		
	}

	j = (file_clpos%512);
	
	for(i=0;i<32;i++) sec_buffer[i+j] = *dat++;
	
	file_pos+=32;
	file_clpos+=32;
	
	if((file_clpos%512)==0)
	{
		ide_writelba(ide_clustertosector(file_cluster)+(file_clpos/512)-1,sec_buffer);
	}	
}

/********************************************************
Berechnet einige globale Variablen die fr 
ide_writedatafile ntig sind. bergeben wird ein String
der den Dateinamen der zu schreibenden Datei beinhalten
mu.
Rckgabe:
1 bei erfogreichem Auffinden der Datei
0 wenn Datei nicht gefunden wurde
*********************************************************/
unsigned char ide_writeinit(unsigned char *fname)
{
	unsigned int i;
	unsigned char *p;
	bit mark = 0;
	
	file_size = 0;
	file_cluster = 0;
	ide_readlba(fat_rootstart,sec_buffer);
	
	for(i=0;i<512;i+=32)
	{
		p=fname;
		
		if(sec_buffer[i+0]!=*p++) continue;
		if(sec_buffer[i+1]!=*p++) continue;
		if(sec_buffer[i+2]!=*p++) continue;
		if(sec_buffer[i+3]!=*p++) continue;
		if(sec_buffer[i+4]!=*p++) continue;
		if(sec_buffer[i+5]!=*p++) continue;
		if(sec_buffer[i+6]!=*p++) continue;
		if(sec_buffer[i+7]!=*p++) continue;
		if(sec_buffer[i+8]!=*p++) continue;
		if(sec_buffer[i+9]!=*p++) continue;
		if(sec_buffer[i+10]!=*p) continue;
		mark = 1;
		break;
	}
	if(mark==1)
	{
		file_cluster = *(unsigned int xdata*) (&sec_buffer[i+26]);
		file_size = *(unsigned long xdata*) (&sec_buffer[i+28]);
		file_pos = 0;
		file_clpos = 0;	
	}

	if((file_cluster!=0)&&(file_size!=0)) return 1; else return 0;
}

/********************************************************
IDE-Reset

Die Reset-Leitung auf dem IDE-Bus wird fr kurze Zeit 
aktiv geschaltet. Danach wird darauf gewartet da das
Master-Device kein Busy mehr meldet

*********************************************************/
void ide_reset(void)
{
	
	P1_0 = 0;
	delay_us(1000);
	P1_0 = 1;
	delay_us(50000);
	
	ide_writecmd(IDE_DH,DEVICE0_LBA);
	
	ide_wbsy();
}

/********************************************************
Fhrt das IDE-Identify-Command aus
*********************************************************/
void ide_readident(void)
{
	unsigned int tmp;
	unsigned char xdata *p = sec_buffer;
	
	ide_wbsy();
	ide_writecmd(IDE_DH,DEVICE0_LBA);
	ide_writecmd(IDE_CMD,IDE_CMD_ID);
	ide_wbsy();

	while(ide_chkdrq())
	{
		tmp = ide_readcmd(IDE_DATA);
		*p++ = tmp>>8;
		*p++ = tmp&0xFF;
	}
	
	return;	
}

/********************************************************
IDE-ReadFile

Eine Datei ab der bergebenen Clusternummer wird gelesen
Der Clusterkette wird korrekt gefolgt.

*********************************************************/
void ide_readfile(unsigned int cluster)
{
	
	unsigned char i = 0;
	unsigned int j = 0;

read_cluster:
	
	for(i=0;i<fat_clustersize;i++)
	{
		j = ide_clustertosector(cluster) + i;
		ide_readlba(j, sec_buffer);
		for(j=0;j<512;j++) ser_putc(sec_buffer[j]);
	}
	
	cluster = ide_getnextcluster(cluster);
	if((cluster&0xFFF8)!=0xFFF8) goto read_cluster;	
}

/********************************************************
IDE-Clustertosector

Berechnet den ersten Sektor eines gegebenen Clusters

*********************************************************/
unsigned int ide_clustertosector(unsigned int cluster)
{
	return fat_datastart + ((cluster - 2) * fat_clustersize);
}


/********************************************************
IDE-getnextcluster

Liefert den nchsten Cluster aus der gegebenen 
Clusterkette

*********************************************************/
unsigned int ide_getnextcluster(unsigned int cluster)
{
	ide_readlba(fat_start + (cluster>>8),sec_buffer);
	return (*(unsigned int xdata*) &sec_buffer[(cluster & 0xFF)*2]);
}

/********************************************************
Initialisiert einige Variablen fr die FAT(erste Partition)
*********************************************************/
void ide_initfat(void)
{
	unsigned long lba_cur;

	ide_readlba(0,sec_buffer);										//MBR sammt Partitionstabelle einlesen
	lba_cur = * (unsigned long xdata *) (&sec_buffer[446+8]);		//Ermittlung des BPB der ersten Partition
	ide_readlba(lba_cur,sec_buffer);								//Einlesen des BPB der ersten Partition

	fat_clustersize = sec_buffer[13];								//gibt die Anzahl Sektoren pro Cluster an
	fat_start = lba_cur + *(unsigned int xdata*) (&sec_buffer[14]);	//ergibt die LBA-Adresse des Beginn der FAT (zur Adresse des BPB werden die res. Sektoren hinzuaddiert)
	fat_lenght = *(unsigned int xdata*) (&sec_buffer[22]);			//ergibt die Lnge einer FAT in Sektoren

	fat_rootstart = fat_start + fat_lenght * sec_buffer[16];		//Beginn des Rootverzeichnis
	fat_datastart = fat_rootstart + (*(unsigned int xdata*) (&sec_buffer[17]))/16; 		//Ab dieser Sektornummer beginnt der Datenbereich.

}

/********************************************************
Schreibt einen gegebenen LBA-Sektor
*********************************************************/
void ide_writelba(unsigned long lba, unsigned char xdata *buf)
{
	unsigned int tmp;
	unsigned int i;
	
	ide_wbsy();
		
	ide_writecmd(IDE_SC,1);
	ide_writecmd(IDE_LBA0,lba);
	ide_writecmd(IDE_LBA8,lba>>8);
	ide_writecmd(IDE_LBA16,lba>>16);
	ide_writecmd(IDE_DH,lba>>24|(DEVICE0_LBA & 0xf0));
	ide_writecmd(IDE_CMD,IDE_CMD_WSR);								//Write Sectors Retry

	ide_wbsy();
	
	for(i=0;i<256;i++)
	{
		tmp = *buf++;
		tmp|=(*buf++)<<8;
		ide_writecmd(IDE_DATA,tmp);
	}
	return;
}

/********************************************************
Liet die IDE-Register des Commandblock
*********************************************************/
void ide_dumpreg(void)
{
	ser_print("\n\rERR: ");	
	ser_printhex(ide_readcmd(IDE_ERR));
	ser_print("\n\rSC : ");	
	ser_printhex(ide_readcmd(IDE_SC));
	ser_print("\n\rSN : ");	
	ser_printhex(ide_readcmd(IDE_SN));
	ser_print("\n\rCL : ");	
	ser_printhex(ide_readcmd(IDE_CL));
	ser_print("\n\rCH : ");	
	ser_printhex(ide_readcmd(IDE_CH));
	ser_print("\n\rDH : ");	
	ser_printhex(ide_readcmd(IDE_DH));
	ser_print("\n\rSTA: ");	
	ser_printhex(ide_readcmd(IDE_STAT));
}

/********************************************************
Liet einen gegebenen LBA-Sektor
*********************************************************/
void ide_readlba(unsigned long lba, unsigned char xdata *buf)
{
	
	unsigned int tmp;
	
	ide_wbsy();

	tmp = lba&0xff;

	ide_writecmd(IDE_LBA0,tmp);
	ide_writecmd(IDE_LBA8,(lba>>8)&0xFF);
	ide_writecmd(IDE_LBA16,(lba>>16)&0xFF);
	ide_writecmd(IDE_LBA24,(lba>>24)|DEVICE0_LBA);
	ide_writecmd(IDE_SC,1);
	ide_writecmd(IDE_CMD,IDE_CMD_RS);				//Read Sectors

	ide_wbsy();
	
	while(ide_chkdrq())
	{
		tmp = ide_readcmd(IDE_DATA);
		*buf++ = tmp&0xFF;
		*buf++ = tmp>>8;
	}
	return;
}

/********************************************************
Initialisiert die Platte
*********************************************************/
void ide_init(void)
{
	
//	unsigned int tmp;
		
	ide_writecnt(IDE_DC,IDE_REG_DC_NOINT);		//Kein INT vom Gert

/*
	ide_readident();	
	fat_ncylinder = *(unsigned int xdata *) (&sec_buffer[108]);
	fat_nheads =  *(unsigned char xdata *) (&sec_buffer[111]);
	fat_nsecptra =  *(unsigned char xdata *) (&sec_buffer[113]);	
	tmp = fat_ncylinder<<8;
	fat_ncylinder = fat_ncylinder>>8;
	fat_ncylinder|=tmp;
*/	
	ide_wbsy();
	
//	ide_writecmd(IDE_SC,fat_nsecptra);
//	ide_writecmd(IDE_DH,DEVICE0_CHS | fat_nheads);
//	ide_writecmd(IDE_CMD,IDE_CMD_IDP);
	
//	ide_wbsy();

}

/********************************************************
Wartet bis das Busy-Bit im Statusregister 0 wird
*********************************************************/
void ide_wbsy(void)
{
	unsigned char tmp = 0xFF;;
	
	P1 = 0xFF;
	
	while(tmp&0x80)
	{	
		IDE_CS0 = 0;
		IDE_RD = 0;
		tmp = IDE_DATL;
		IDE_RD = 1;
		IDE_CS0 = 1;
	}
}

/********************************************************
Liefert den Status des DRQ-Bit im Statusregister
*********************************************************/
unsigned char ide_chkdrq(void)
{
	unsigned char tmp;
	
	P1 = 0xFF;
	
	IDE_CS0 = 0;
	IDE_RD = 0;
	tmp = IDE_DATL;
	IDE_RD = 1;
	IDE_CS0 = 1;
	
	if((tmp&0x08)==0x08) return 1;
	else return 0;
}

/********************************************************
Liet aus dem gegebenen Register im Command-Block
*********************************************************/
unsigned int ide_readcmd(unsigned char adr)
{
	unsigned int res;
	
	adr = adr<<5;
	P1&=0x1F;
	P1|=adr;
	IDE_CS0 = 0;
	IDE_RD = 0;
	res = IDE_DATL;
	res|= IDE_DATH<<8;
	IDE_RD = 1;
	IDE_CS0 = 1;
	P1|=0xE0;
	
	return res;
}

/********************************************************
Liet aus dem gegebenen Register im Control-Block
*********************************************************/
unsigned int ide_readcnt(unsigned char adr)
{
	unsigned int res;
	
	adr = adr<<5;
	P1&=0x1F;
	P1|=adr;
	IDE_CS1 = 0;
	IDE_RD = 0;
	res = IDE_DATL;
	res|= IDE_DATH<<8;
	IDE_RD = 1;
	IDE_CS1 = 1;
	P1|=0xE0;
	
	return res;
}

/********************************************************
Schreibt in das gegebene Register im Command-Block
*********************************************************/
void ide_writecmd(unsigned char adr, unsigned int dat)
{
	adr = adr<<5;
	P1&=0x1F;
	P1|=adr;
	IDE_DATL = dat&0xFF;
	//IDE_DATH = (dat>>8)&0xFF;
	IDE_DATH = dat>>8;
	IDE_CS0 = 0;
	IDE_WR = 0;
	IDE_WR = 1;
	IDE_CS0 = 1;
	P1|=0xE0;
	IDE_DATL = 0xFF;
	IDE_DATH = 0xFF;
}

/********************************************************
Schreibt in das gegebene Register im Control-Block
*********************************************************/
void ide_writecnt(unsigned char adr, unsigned int dat)
{
	adr = adr<<5;
	P1&=0x1F;
	P1|=adr;
	IDE_DATL = dat&0xFF;
	IDE_DATH = (dat>>8)&0xFF;
	IDE_CS1 = 0;
	IDE_WR = 0;
	IDE_WR = 1;
	IDE_CS1 = 1;
	P1|=0xE0;
	IDE_DATL = 0xFF;
	IDE_DATH = 0xFF;
}


