#ifndef __IDE_H
#define __IDE_H

//ATA-Commands

#define IDE_CMD_CPM			0xE5			//Typ 3 Check Power Mode
#define IDE_CMD_EDD			0x90			//Typ 3 Execute Device Diagnostic
#define IDE_CMD_FT			0x50			//Typ 2 Format Track
#define IDE_CMD_ID			0xEC			//Typ 1 Identify Device
#define IDE_CMD_IDDMA		0xEE			//Typ 4 Identify Device DMA
#define IDE_CMD_IDL			0xE3			//Typ 3 Idle
#define IDE_CMD_IDLE		0xE1			//Typ 3 Idle Immediate
#define IDE_CMD_IDP			0x91			//Typ 3 Initialize Device Parameters
#define IDE_CMD_RB			0xE4			//Typ 1 Read Buffer
#define IDE_CMD_RDMAR		0xC8			//Typ 4 Read DMA (retry)
#define IDE_CMD_RDMA		0xC9			//Typ 4 Read DMA (no retry)
#define IDE_CMD_RLR			0x22			//Typ 1 Read Long (retry)
#define IDE_CMD_RL			0x23			//Typ 1 Read Long (no retry)
#define IDE_CMD_RM			0xC4			//Typ 1 Read Multiple
#define IDE_CMD_RSR			0x20			//Typ 1 Read Sectors (retry)
#define IDE_CMD_RS			0x21			//Typ 1 Read Sectors (no retry)
#define IDE_CMD_RVSR		0x40			//Typ 3 Read Verify Sectors (retry)
#define IDE_CMD_RVS			0x41			//Typ 3 Read Verify Sectors (no retry)
#define IDE_CMD_RE			0x10			//Typ 3 Recalibrate (0x1x)
#define IDE_CMD_SDP			0xF6			//Typ 2 Security Disable Password
#define IDE_CMD_SEP			0xF3			//Typ 3 Security Erase Prepare
#define IDE_CMD_SEU			0xF4			//Typ 2 Security Erase Unit
#define IDE_CMD_SFL			0xF5			//Typ 3 Security Freeze Lock
#define IDE_CMD_SSP			0xF1			//Typ 2 Security Set Password
#define IDE_CMD_SU			0xF2			//Typ 2 Security Unlock
#define IDE_CMD_SE			0x70			//Typ 3 Seek (0x7x)
#define IDE_CMD_SF			0xEF			//Typ 3 Set Features
#define IDE_CMD_SMM			0xC6			//Typ 3 Set Multiple Mode
#define IDE_CMD_SL			0xE6			//Typ 3 Sleep
#define IDE_CMD_SMART_DO	0xB0			//Typ 3 SMART Disable Operations		(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_EDAA	0xB0			//Typ 3 SMART Enable/Disable Attribute Autosave (braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_EO	0xB0			//Typ 3 SMART Enable Operations			(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_EODC	0xB0			//Typ 3 SMART Execute Offline Data Collection	(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_RAV	0xB0			//Typ 1 SMART Read Attribute Values		(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_RAT	0xB0			//Typ 1 SMART Read Attribute Thresholds		(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_RS	0xB0			//Typ 3 SMART Return Status			(braucht Subcommand in Feature-Register)
#define IDE_CMD_SMART_SAV	0xB0			//Typ 3 SMART Save Attribute Values		(braucht Subcommand in Feature-Register)
#define IDE_CMD_SB			0xE2			//Typ 3 Standby
#define IDE_CMD_SBI			0xE0			//Typ 3 Standby Immediate
#define IDE_CMD_WB			0xE8			//Typ 2 Write Buffer
#define IDE_CMD_WDMAR		0xCA			//Typ 4 Write DMA (retry)
#define IDE_CMD_WDMA		0xCB			//Typ 4 Write DMA (no retry)
#define IDE_CMD_WLR			0x32			//Typ 2 Write Long (retry)
#define IDE_CMD_WL			0x33			//Typ 2 Write Long (no retry)
#define IDE_CMD_WM			0xC5			//Typ 2 Write Multiple
#define IDE_CMD_WSR			0x30			//Typ 2 Write Sectors (retry)
#define IDE_CMD_WS			0x31			//Typ 2 Write Sectors (no retry)
#define IDE_CMD_WV			0x3C			//Typ 2 Write Verify

#define IDE_DATA			0x00			// data				in/out
#define IDE_ERR				0x01 			// error            in		
#define IDE_FR	    		0x01 			// feature reg         out	
#define IDE_SC	    		0x02 			// sector count     in/out	
#define IDE_SN	    		0x03 			// sector number    in/out	
#define IDE_CL	    		0x04 			// cylinder low     in/out	
#define IDE_CH	    		0x05 			// cylinder high    in/out	
#define IDE_DH	    		0x06 			// device head      in/out	
#define IDE_STAT			0x07 			// status   	    in		
#define IDE_CMD			    0x07 			// command             out	

//IDE-Register Control Block
#define IDE_ASTAT			0x06 			// alternate status in		        
#define IDE_DC	    		0x06 			// device control      out	 

#define IDE_LBA0			0x03			//Definitionen fr die LBA-Register
#define IDE_LBA8			0x04
#define IDE_LBA16			0x05
#define IDE_LBA24			0x06


#define DEVICE0_CHS			0xA0
#define DEVICE0_LBA			0xE0
#define DEVICE1_CHS			0xB0
#define DEVICE1_LBA			0xF0


/*
#define DEVICE1_CHS			0xA0
#define DEVICE1_LBA			0xE0
#define DEVICE0_CHS			0xB0
#define DEVICE0_LBA			0xF0
*/
	
#define IDE_REG_DC_NOINT	0x0A			//Device-Control-Register: kein INT

#define FILE_ATTR_READ_ONLY 0x01
#define FILE_ATTR_HIDDEN 	0x02
#define FILE_ATTR_SYSTEM 	0x04
#define FILE_ATTR_VOLUME_ID 0x08
#define FILE_ATTR_DIRECTORY 0x10
#define FILE_ATTR_ARCHIVE 	0x20

#define FILE_ATTR_LFN		0x0F

#define IDE_RESET	P1_0
#define IDE_WR		P1_1
#define IDE_RD		P1_2
#define IDE_CS0		P1_3
#define IDE_CS1		P1_4
#define IDE_A0		P1_5
#define IDE_A1		P1_6
#define IDE_A2		P1_7

#define IDE_DATL	P2
#define IDE_DATH	P0


//Funktionsprototypen
void ide_reset(void);										//Hardreset der Platte/CF
unsigned char ide_writeinit(unsigned char *fname);			//Initialsierung fr Schreibvariablen
void ide_readident(void);									//Fhrt das Identify-cmd aus
void ide_initfat(void);										//Initialisierung der Variablen fr die FAT16
unsigned char ide_writedatafile(unsigned char xdata *dat);	//Schreibt Daten zu je 32Byte auf die Platte (512Byte Puffer)
void ide_readfile(unsigned int cluster);					//Liet eine Datei ab angegebenen Cluster
void ide_writelba(unsigned long lba, unsigned char xdata *buf);//Schreibt einen Sektor auf die Platte
unsigned int ide_clustertosector(unsigned int cluster);		//Rechnet eine Clusternummer in eine Sektornummer
unsigned int ide_getnextcluster(unsigned int cluster);		//Besorgt die nchste Clusternummer aus der FAT
void ide_readlba(unsigned long lba, unsigned char xdata *buf);//Liet einen Sektor von der Platte
void ide_init(void);										//Initialisiert die Platte/CF
void ide_wbsy(void);										//Wartet auf des Busybit der Platte
unsigned char ide_chkdrq(void);								//Liefert den Wert des DRQ-Flags
unsigned int ide_readcmd(unsigned char adr);				//Liet aus dem Command-Block
unsigned int ide_readcnt(unsigned char adr);				//Liet aus dem Control-Block
void ide_writecmd(unsigned char adr, unsigned int data);	//Schreibt in den Command-Block
void ide_writecnt(unsigned char adr, unsigned int data);	//Schreibt in den Control-Block

void ide_dumpreg(void);										//Gibt die IDE-Register auf der seriellen aus


#endif
